@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -21,10 +21,11 @@ import os
 import re
 import binascii
 import threading
+from hashlib import sha1
 from contextlib import contextmanager
 
 from hypothesis._settings import note_deprecation
-from hypothesis.internal.compat import FileNotFoundError, sha1, hbytes, \
+from hypothesis.internal.compat import FileNotFoundError, hbytes, \
     b64decode, b64encode
 
 sqlite3 = None
@@ -59,15 +60,13 @@ class ExampleDatabase(EDMeta('ExampleDatabase', (object,), {})):
     A key -> multiple distinct values mapping.
 
     Keys and values are binary data.
-
     """
 
     def save(self, key, value):
-        """save this value under this key.
+        """Save ``value`` under ``key``.
 
         If this value is already present for this key, silently do
         nothing
-
         """
         raise NotImplementedError('%s.save' % (type(self).__name__))
 
@@ -75,7 +74,6 @@ class ExampleDatabase(EDMeta('ExampleDatabase', (object,), {})):
         """Remove this value from this key.
 
         If this value is not present, silently do nothing.
-
         """
         raise NotImplementedError('%s.delete' % (type(self).__name__))
 
@@ -86,7 +84,6 @@ class ExampleDatabase(EDMeta('ExampleDatabase', (object,), {})):
 
         Note that value will be inserted at dest regardless of whether
         it is currently present at src.
-
         """
         if src == dest:
             self.save(src, value)
