@@ -35,12 +35,13 @@ from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
     text_type, get_type_hints, getfullargspec, implements_iterator
 from hypothesis.internal.floats import is_negative, float_to_int, \
     int_to_float, count_between_floats
+from hypothesis.internal.charmap import as_general_categories
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
 from hypothesis.internal.validation import check_type, try_convert, \
-    check_strategy, check_valid_bound, check_valid_sizes, \
-    check_valid_integer, check_valid_interval
+    check_strategy, check_valid_size, check_valid_bound, \
+    check_valid_sizes, check_valid_integer, check_valid_interval
 
 __all__ = [
     'nothing',
@@ -117,7 +118,15 @@ def cacheable(fn):
 
 
 def base_defines_strategy(force_reusable):
+    """Returns a decorator for strategy functions.
+
+    If force_reusable is True, the generated values are assumed to be
+    reusable, i.e. immutable and safe to cache, across multiple test
+    invocations.
+    """
     def decorator(strategy_definition):
+        """A decorator that registers the function as a strategy and makes it
+        lazily evaluated."""
         from hypothesis.searchstrategy.lazy import LazyStrategy
         _strategies.add(strategy_definition.__name__)
 
@@ -170,7 +179,6 @@ def nothing():
     an attempt to draw.
 
     Examples from this strategy do not shrink (because there are none).
-
     """
     return NOTHING
 
@@ -185,20 +193,18 @@ def just(value):
     of ``just(callable())`` to get a fresh value each time.
 
     Examples from this strategy do not shrink (because there is only one).
-
     """
     from hypothesis.searchstrategy.misc import JustStrategy
 
     return JustStrategy(value)
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 def none():
     """Return a strategy which only generates None.
 
     Examples from this strategy do not shrink (because there is only
     one).
-
     """
     return just(None)
 
@@ -221,7 +227,6 @@ def one_of(*args):
     ``x = st.deferred(lambda: st.none() | st.tuples(x, x))`` will shrink well,
     but ``x = st.deferred(lambda: st.tuples(x, x) | st.none())`` will shrink
     very badly indeed.
-
     """
     if len(args) == 1 and not isinstance(args[0], SearchStrategy):
         try:
@@ -243,7 +248,6 @@ def integers(min_value=None, max_value=None):
 
     Examples from this strategy will shrink towards zero, and negative values
     will also shrink towards positive (i.e. -n may be replaced by +n).
-
     """
 
     check_valid_bound(min_value, 'min_value')
@@ -293,7 +297,6 @@ def booleans():
 
     Examples from this strategy will shrink towards False (i.e.
     shrinking will try to replace True with False where possible).
-
     """
     from hypothesis.searchstrategy.misc import BoolStrategy
     return BoolStrategy()
@@ -320,7 +323,6 @@ def floats(
     shrinking behaviour, but it tries to improve "human readability". Finite
     numbers will be preferred to infinity and infinity will be preferred to
     NaN.
-
     """
 
     if allow_nan is None:
@@ -420,7 +422,6 @@ def complex_numbers():
 
     Examples from this strategy shrink by shrinking their component real
     and imaginary parts.
-
     """
     from hypothesis.searchstrategy.numbers import ComplexStrategy
     return ComplexStrategy(
@@ -438,13 +439,12 @@ def tuples(*args):
     two with both values an integer.
 
     Examples from this strategy shrink by shrinking their component parts.
-
     """
     for arg in args:
         check_strategy(arg)
 
     from hypothesis.searchstrategy.collections import TupleStrategy
-    return TupleStrategy(args, tuple)
+    return TupleStrategy(args)
 
 
 @defines_strategy
@@ -462,7 +462,6 @@ def sampled_from(elements):
     the list. So e.g. sampled_from((10, 1)) will shrink by trying to replace
     1 values with 10, and sampled_from((1, 10)) will shrink by trying to
     replace 10 values with 1.
-
     """
     from hypothesis.searchstrategy.misc import SampledFromStrategy
     from hypothesis.internal.conjecture.utils import check_sample
@@ -508,13 +507,21 @@ def lists(
 
     Examples from this strategy shrink by trying to remove elements from the
     list, and by shrinking each individual element of the list.
-
     """
     check_valid_sizes(min_size, average_size, max_size)
-    if elements is None or (max_size is not None and max_size <= 0):
-        if max_size is None or max_size > 0:
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the list will be required '
+            'in a future version of Hypothesis.  To create lists that are '
+            'always empty, use `builds(list)` or `lists(nothing())`.'
+        )
+        if min_size or average_size or max_size:
+            # Checked internally for lists with an elements strategy, but
+            # we're about to skip that and return builds(list) instead...
             raise InvalidArgument(
-                u'Cannot create non-empty lists without an element type'
+                'Cannot create a non-empty collection (min_size=%r, '
+                'average_size=%r, max_size=%r) without elements.'
+                % (min_size, average_size, max_size)
             )
         else:
             return builds(list)
@@ -555,7 +562,7 @@ def lists(
         )
     else:
         return ListStrategy(
-            (elements,), average_length=average_size,
+            elements, average_size=average_size,
             min_size=min_size, max_size=max_size,
         )
 
@@ -571,8 +578,13 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 
     Examples from this strategy shrink by trying to remove elements from the
     set, and by shrinking each individual element of the set.
-
     """
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the set will be required '
+            'in a future version of Hypothesis.  To create sets that are '
+            'always empty, use `builds(set)` or `sets(nothing())`.'
+        )
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
         max_size=max_size, unique=True
@@ -584,6 +596,13 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 def frozensets(elements=None, min_size=None, average_size=None, max_size=None):
     """This is identical to the sets function but instead returns
     frozensets."""
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the frozenset will be '
+            'required in a future version of Hypothesis.  To create '
+            'frozensets that are always empty, use `builds(frozenset)` '
+            'or `frozensets(nothing())`.'
+        )
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
         max_size=max_size, unique=True
@@ -599,8 +618,14 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
     fixed length (e.g. generators). This strategy produces iterators,
     which cannot be indexed and do not have a fixed length. This ensures
     that you do not accidentally depend on sequence behaviour.
-
     """
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the iterable will be '
+            'required in a future version of Hypothesis.  To create '
+            'iterables that are always empty, use `iterables(nothing())`.'
+        )
+
     @implements_iterator
     class PrettyIter(object):
         def __init__(self, values):
@@ -634,7 +659,6 @@ def fixed_dictionaries(mapping):
 
     Examples from this strategy shrink by shrinking each individual value in
     the generated dictionary.
-
     """
     from hypothesis.searchstrategy.collections import FixedKeysDictStrategy
     check_type(dict, mapping, 'mapping')
@@ -656,7 +680,6 @@ def dictionaries(
 
     Examples from this strategy shrink by trying to remove keys from the
     generated dictionary, and by shrinking each generated key and value.
-
     """
     check_valid_sizes(min_size, average_size, max_size)
     if max_size == 0:
@@ -683,7 +706,6 @@ def streaming(elements):
 
     .. deprecated:: 3.15.0
         Use :func:`data() <hypothesis.strategies.data>` instead.
-
     """
     note_deprecation(
         'streaming() has been deprecated. Use the data() strategy instead and '
@@ -704,39 +726,43 @@ def characters(whitelist_categories=None, blacklist_categories=None,
     """Generates unicode text type (unicode on python 2, str on python 3)
     characters following specified filtering rules.
 
-    When no filtering rules are specifed, any character can be produced.
+    - When no filtering rules are specifed, any character can be produced.
+    - If ``min_codepoint`` or ``max_codepoint`` is specifed, then only
+      characters having a codepoint in that range will be produced.
+    - If ``whitelist_categories`` is specified, then only characters from those
+      Unicode categories will be produced. This is a further restriction,
+      characters must also satisfy ``min_codepoint`` and ``max_codepoint``.
+    - If ``blacklist_categories`` is specified, then any character from those
+      categories will not be produced.  Any overlap between
+      ``whitelist_categories`` and ``blacklist_categories`` will raise an
+      exception, as each character can only belong to a single class.
+    - If ``whitelist_characters`` is specified, then any additional characters
+      in that list will also be produced.
+    - If ``blacklist_characters`` is specified, then any characters in
+      that list will be not be produced. Any overlap between \
+      ``whitelist_characters`` and ``blacklist_characters`` will raise an
+      exception.
 
-    If ``min_codepoint`` or ``max_codepoint`` is specifed, then only
-    characters having a codepoint in that range will be produced.
+    The ``_codepoint`` arguments must be integers between zero and
+    :obj:`python:sys.max_unicode`.  The ``_characters`` arguments must be
+    collections of length-one unicode strings, such as a unicode string.
 
-    If ``whitelist_categories`` is specified, then only characters from those
-    Unicode categories will be produced. This is a further restriction,
-    characters must also satisfy ``min_codepoint`` and ``max_codepoint``.
+    The ``_categories`` arguments must be used to specify either the
+    one-letter Unicode major category or the two-letter Unicode
+    `general category`_.  For example, ``('Nd', 'Lu')`` signifies "Number,
+    decimal digit" and "Letter, uppercase".  A single letter ('major category')
+    can be given to match all corresponding categories, for example ``'P'``
+    for characters in any punctuation category.
 
-    If ``blacklist_categories`` is specified, then any character from those
-    categories will not be produced. This is a further restriction,
-    characters that match both ``whitelist_categories`` and
-    ``blacklist_categories`` will not be produced.
-
-    If ``whitelist_characters`` is specified, then any additional characters
-    in that list will also be produced.
-
-    If ``blacklist_characters`` is specified, then any characters in that list
-    will be not be produced. Any overlap between ``whitelist_characters`` and
-    ``blacklist_characters`` will raise an exception.
-
-    Examples from this strategy shrink towards smaller codepoints.
+    .. _general category: https://wikipedia.org/wiki/Unicode_character_property
 
+    Examples from this strategy shrink towards the codepoint for ``'0'``,
+    or the first allowable codepoint after it if ``'0'`` is excluded.
     """
-    if (
-        min_codepoint is not None and max_codepoint is not None and
-        min_codepoint > max_codepoint
-    ):
-        raise InvalidArgument(
-            'Cannot have min_codepoint=%d > max_codepoint=%d ' % (
-                min_codepoint, max_codepoint
-            )
-        )
+    check_valid_size(min_codepoint, 'min_codepoint')
+    check_valid_size(max_codepoint, 'max_codepoint')
+    check_valid_interval(min_codepoint, max_codepoint,
+                         'min_codepoint', 'max_codepoint')
     if all((whitelist_characters is not None,
             min_codepoint is None,
             max_codepoint is None,
@@ -744,25 +770,33 @@ def characters(whitelist_categories=None, blacklist_categories=None,
             blacklist_categories is None,
             )):
         raise InvalidArgument(
-            'Cannot have just whitelist_characters=%r alone, '
-            'it would have no effect. Perhaps you want sampled_from()' % (
-                whitelist_characters,
-            )
-        )
-    if (
-        whitelist_characters is not None and
-        blacklist_characters is not None and
-        set(blacklist_characters).intersection(set(whitelist_characters))
-    ):
+            'Passing only whitelist_characters=%r would have no effect. '
+            'Perhaps you want sampled_from() ?' % (whitelist_characters,))
+    blacklist_characters = blacklist_characters or ''
+    whitelist_characters = whitelist_characters or ''
+    overlap = set(blacklist_characters).intersection(whitelist_characters)
+    if overlap:
         raise InvalidArgument(
             'Characters %r are present in both whitelist_characters=%r, and '
             'blacklist_characters=%r' % (
-                set(blacklist_characters).intersection(
-                    set(whitelist_characters)
-                ),
-                whitelist_characters, blacklist_characters,
-            )
-        )
+                sorted(overlap), whitelist_characters, blacklist_characters))
+    blacklist_categories = as_general_categories(
+        blacklist_categories, 'blacklist_categories')
+    if whitelist_categories is not None and not whitelist_categories and \
+            not whitelist_characters:
+        raise InvalidArgument(
+            'When whitelist_categories is an empty collection and there are '
+            'no characters specified in whitelist_characters, nothing can '
+            'be generated by the characters() strategy.')
+    whitelist_categories = as_general_categories(
+        whitelist_categories, 'whitelist_categories')
+    both_cats = set(
+        blacklist_categories or ()).intersection(whitelist_categories or ())
+    if both_cats:
+        raise InvalidArgument(
+            'Categories %r are present in both whitelist_categories=%r, and '
+            'blacklist_categories=%r' % (
+                sorted(both_cats), whitelist_categories, blacklist_categories))
 
     from hypothesis.searchstrategy.strings import OneCharStringStrategy
     return OneCharStringStrategy(whitelist_categories=whitelist_categories,
@@ -790,7 +824,6 @@ def text(
 
     Examples from this strategy shrink towards shorter strings, and with the
     characters in the text shrinking as per the alphabet strategy.
-
     """
     from hypothesis.searchstrategy.strings import StringStrategy
     if alphabet is None:
@@ -840,7 +873,6 @@ def from_regex(regex):
 
     Examples from this strategy shrink towards shorter strings and lower
     character values.
-
     """
     from hypothesis.searchstrategy.regex import regex_strategy
     return regex_strategy(regex)
@@ -858,7 +890,6 @@ def binary(
 
     Examples from this strategy shrink towards smaller strings and lower byte
     values.
-
     """
     from hypothesis.searchstrategy.strings import BinaryStringStrategy, \
         FixedSizeBytes
@@ -880,7 +911,6 @@ def randoms():
     RandomWithSeed class which displays what it was initially seeded with)
 
     Examples from this strategy shrink to seeds closer to zero.
-
     """
     from hypothesis.searchstrategy.misc import RandomStrategy
     return RandomStrategy(integers())
@@ -909,7 +939,6 @@ def random_module():
     which will give you one.
 
     Examples from these strategy shrink to seeds closer to zero.
-
     """
     from hypothesis.control import cleanup
     import random
@@ -928,23 +957,40 @@ def random_module():
 
 @cacheable
 @defines_strategy
-def builds(target, *args, **kwargs):
+def builds(*callable_and_args, **kwargs):
     """Generates values by drawing from ``args`` and ``kwargs`` and passing
-    them to ``target`` in the appropriate argument position.
+    them to the callable (provided as the first positional argument) in the
+    appropriate argument position.
 
     e.g. ``builds(target, integers(), flag=booleans())`` would draw an
     integer ``i`` and a boolean ``b`` and call ``target(i, flag=b)``.
 
-    If ``target`` has type annotations, they will be used to infer a strategy
+    If the callable has type annotations, they will be used to infer a strategy
     for required arguments that were not passed to builds.  You can also tell
     builds to infer a strategy for an optional argument by passing the special
     value :const:`hypothesis.infer` as a keyword argument to
-    builds, instead of a strategy for that argument to ``target``.
+    builds, instead of a strategy for that argument to the callable.
 
     Examples from this strategy shrink by shrinking the argument values to
-    the target.
-
+    the callable.
     """
+    if callable_and_args:
+        target, args = callable_and_args[0], callable_and_args[1:]
+        if not callable(target):
+            raise InvalidArgument(
+                'The first positional argument to builds() must be a callable '
+                'target to construct.')
+    elif 'target' in kwargs and callable(kwargs['target']):
+        args = []
+        note_deprecation(
+            'Specifying the target as a keyword argument to builds() is '
+            'deprecated. Provide it as the first positional argument instead.')
+        target = kwargs.pop('target')
+    else:
+        raise InvalidArgument(
+            'builds() must be passed a callable as the first positional '
+            'argument, but no positional arguments were given.')
+
     if infer in args:
         # Avoid an implementation nightmare juggling tuples and worse things
         raise InvalidArgument('infer was passed as a positional argument to '
@@ -999,7 +1045,6 @@ def from_type(thing):
        other elements in the lookup.
     4. Finally, if ``thing`` has type annotations for all required arguments,
        it is resolved via :func:`~hypothesis.strategies.builds`.
-
     """
     from hypothesis.searchstrategy import types
     try:
@@ -1107,7 +1152,6 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
 
     Examples from this strategy shrink towards smaller denominators, then
     closer to zero.
-
     """
     min_value = try_convert(Fraction, min_value, 'min_value')
     max_value = try_convert(Fraction, max_value, 'max_value')
@@ -1213,7 +1257,6 @@ def decimals(min_value=None, max_value=None,
 
     Examples from this strategy do not have a well defined shrink order but
     try to maximize human readability when shrinking.
-
     """
     # Convert min_value and max_value to Decimal values, and validate args
     check_valid_integer(places)
@@ -1323,7 +1366,6 @@ def permutations(values):
 
     Examples from this strategy shrink by trying to become closer to the
     original order of values.
-
     """
     from hypothesis.internal.conjecture.utils import integer_range
 
@@ -1381,7 +1423,6 @@ def datetimes(
     are at the beginning of the list for proper minimisation.
 
     Examples from this strategy shrink towards midnight on January 1st 2000.
-
     """
     # Why must bounds be naive?  In principle, we could also write a strategy
     # that took aware bounds, but the API and validation is much harder.
@@ -1423,7 +1464,6 @@ def dates(
     """A strategy for dates between ``min_date`` and ``max_date``.
 
     Examples from this strategy shrink towards January 1st 2000.
-
     """
     from hypothesis.searchstrategy.datetime import DateStrategy
 
@@ -1450,7 +1490,6 @@ def times(
 
     Examples from this strategy shrink towards midnight, with the timezone
     component shrinking as for the strategy that provided it.
-
     """
     check_type(dt.time, min_value, 'min_value')
     check_type(dt.time, max_value, 'max_value')
@@ -1477,7 +1516,6 @@ def timedeltas(
     """A strategy for timedeltas between ``min_value`` and ``max_value``.
 
     Examples from this strategy shrink towards zero.
-
     """
     from hypothesis.searchstrategy.datetime import TimedeltaStrategy
 
@@ -1500,10 +1538,10 @@ def composite(f):
 
     Examples from this strategy shrink by shrinking the output of each draw
     call.
-
     """
 
     from hypothesis.internal.reflection import define_function_signature
+    from hypothesis.internal.conjecture.utils import calc_label_from_cls
     argspec = getfullargspec(f)
 
     if (
@@ -1522,6 +1560,8 @@ def composite(f):
               if k in (argspec.args + argspec.kwonlyargs + ['return'])}
     new_argspec = argspec._replace(args=argspec.args[1:], annotations=annots)
 
+    label = calc_label_from_cls(f)
+
     @defines_strategy
     @define_function_signature(f.__name__, f.__doc__, new_argspec)
     def accept(*args, **kwargs):
@@ -1535,6 +1575,10 @@ def composite(f):
                     return data.draw(strategy)
 
                 return f(draw, *args, **kwargs)
+
+            @property
+            def label(self):
+                return label
         return CompositeStrategy()
     accept.__module__ = f.__module__
     return accept
@@ -1556,7 +1600,6 @@ def shared(base, key=None):
     >>> y = shared(s, key="hi")
 
     Examples from this strategy shrink as per their base strategy.
-
     """
     from hypothesis.searchstrategy.shared import SharedStrategy
     return SharedStrategy(base, key)
@@ -1575,7 +1618,6 @@ def choices():
 
     Examples from this strategy shrink by making each choice function return
     an earlier value in the sequence passed to it.
-
     """
     from hypothesis.control import note, current_build_context
     from hypothesis.internal.conjecture.utils import choice, check_sample
@@ -1630,7 +1672,6 @@ def uuids(version=None):
     ``lists(uuids())`` the resulting list will never contain duplicates.
 
     Examples from this strategy don't have any meaningful shrink order.
-
     """
     from uuid import UUID
     if version not in (None, 1, 2, 3, 4, 5):
@@ -1653,7 +1694,6 @@ def runner(default=not_set):
     that default. If no default is provided, raises InvalidArgument.
 
     Examples from this strategy do not shrink (because there is only one).
-
     """
     class RunnerStrategy(SearchStrategy):
 
@@ -1686,7 +1726,6 @@ def data():
 
     Examples from this strategy do not shrink (because there is only one),
     but the result of calls to each draw() call shrink as they normally would.
-
     """
     from hypothesis.control import note
 
@@ -1755,7 +1794,6 @@ def register_type_strategy(custom_type, strategy):
 
     ``strategy`` may be a search strategy, or a function that takes a type and
     returns a strategy (useful for generic types).
-
     """
     from hypothesis.searchstrategy import types
     if not isinstance(custom_type, type):
@@ -1800,7 +1838,6 @@ def deferred(definition):
 
     Examples from this strategy shrink as they normally would from the strategy
     returned by the definition.
-
     """
     from hypothesis.searchstrategy.deferred import DeferredStrategy
     return DeferredStrategy(definition)
